@@ -3,8 +3,8 @@
 plotHive <- function(HPD, ch = 1, method = "abs",
 	dr.nodes = TRUE, bkgnd = "black",
 	axLabs = NULL, axLab.pos = NULL, axLab.gpar = NULL,
-	anNodes = NULL, anNode.gpar = NULL,
-	arrow = NULL, np = TRUE, ...) {
+	anNodes = NULL, anNode.gpar = NULL, grInfo = NULL,
+	arrow = NULL, np = TRUE, anCoord = "local", ...) {
 	
 	# Function to plot hive plots using grid graphics
 	# Inspired by the work of Martin Kryzwinski
@@ -81,49 +81,104 @@ plotHive <- function(HPD, ch = 1, method = "abs",
 		grid.text(arrow[1], x.lab, y.lab, default.units = "native", gp = axLab.gpar)
 		}
 
-	annotateNodes <- function(anNodes, nodes, nx) {
+	annotateNodes <- function(anNodes, nodes, nx, anCoord) {
 
 		if (is.null(anNode.gpar)) {
 			if (bkgnd == "black") anNode.gpar <- gpar(fontsize = 10, col = "white", lwd = 0.5)
 			if (!bkgnd == "black") anNode.gpar <- gpar(fontsize = 10, col = "black", lwd = 0.5)
 			}
 			
-		ann <- read.csv(anNodes, header = TRUE)
-		# Columns should be: node.lab, node.text, angle, radius, offset
-		# Locate the node on an axis at a particular radius
+		ann <- read.csv(anNodes, header = TRUE, colClasses = c(rep("character", 2), rep("numeric", 5)))
+		cds <- getCoords(anNodes, anCoord, nodes)
 		
-		id <- c()	
-		for (n in 1:nrow(ann)) {			
-			pat <- paste("\\b", ann$node.lab[n], "\\b", sep = "") 
-			id <- c(id, grep(pat, nodes$lab))
+		grid.segments(x0 = cds$x.st, x1 = cds$x.end, y0 = cds$y.st, y1 = cds$y.end,
+			default.units = "native", gp = anNode.gpar)
+		grid.text(ann$node.text, cds$x.lab, cds$y.lab, hjust = ann$hjust, vjust = ann$vjust,
+			default.units = "native", gp = anNode.gpar)
+		}
+
+	addGraphic <- function(grInfo, nodes, nx, anCoord) {
+
+		gr <- read.csv(grInfo, header = TRUE, stringsAsFactors = FALSE)
+		cds <- getCoords(grInfo, anCoord, nodes)
+				
+		grid.segments(x0 = cds$x.st, x1 = cds$x.end, y0 = cds$y.st, y1 = cds$y.end,
+			default.units = "native", gp = anNode.gpar)
+			
+		# readJPEG and readPNG are not vectorized, grab each graphic in turn
+		# Figure out if we are using jpg or png files
+		
+		ext <- substr(gr$path[1], nchar(gr$path[1])-2, nchar(gr$path[1]))
+		if ((ext == "png") | (ext == "PNG")) ext <- "png"
+		if ((ext == "jpg") | (ext == "JPG") | (ext == "peg") | (ext =="PEG")) ext <- "jpg"
+
+		# Now draw the images
+		
+		if (ext == "jpg") {
+			for (n in 1:nrow(gr)) {
+				grid.raster(readJPEG(gr$path[n]),
+					x = cds$x.lab[n], y = cds$y.lab[n], default.units = "native", width = gr$width[n])			
+				}
+			}
+
+		if (ext == "png") {
+			for (n in 1:nrow(gr)) {
+				grid.raster(readPNG(gr$path[n]),
+					x = cds$x.lab[n], y = cds$y.lab[n], default.units = "native", width = gr$width[n])			
+				}
+			}
+
+		}
+
+	getCoords <- function(file, anCoord, nodes) {
+		
+		# Figure out the coordinates of the line segments and labels/graphics
+		# anNodes and grInfo both contains certain columns which are used here
+		
+		df <- read.csv(file, header = TRUE)
+		
+		id <- rep(NA, nrow(df))	
+		for (n in 1:nrow(df)) {			
+			pat <- paste("\\b", df$node.lab[n], "\\b", sep = "") 
+			id[n] <- grep(pat, nodes$lab)
 			}
 		
-			N <- matrix(data = c(
-			0, 180, NA, NA, NA, NA,
-			90, 210, 330, NA, NA, NA,
-			90, 180, 270, 0, NA, NA,
-			90, 162, 234, 306, 18, NA,
-			90, 150, 210, 270, 330, 390),
-			byrow = TRUE, nrow = 5)
+		N <- matrix(data = c(
+		0, 180, NA, NA, NA, NA,
+		90, 210, 330, NA, NA, NA,
+		90, 180, 270, 0, NA, NA,
+		90, 162, 234, 306, 18, NA,
+		90, 150, 210, 270, 330, 390),
+		byrow = TRUE, nrow = 5)
 			
 		ax <- nodes$axis[id] # axis number
 		for (n in 1:length(ax)) {
 			ax[n] <- N[nx-1,ax[n]]			
 			}
 
+		# Figure coords in requested reference frame
+
 		x.st <- p2cX(nodes$radius[id], ax)
 		y.st <- p2cY(nodes$radius[id], ax)
 		
-		x.end <- p2cX(ann$radius, ann$angle)
-		y.end <- p2cY(ann$radius, ann$angle)
-					
-		x.lab <- p2cX(ann$radius + ann$offset, ann$angle) # figure label position
-		y.lab <- p2cY(ann$radius + ann$offset, ann$angle)
+		if (anCoord == "local") {
+			x.end <- x.st + p2cX(df$radius, df$angle)
+			y.end <- y.st + p2cY(df$radius, df$angle)
+						
+			x.lab <- x.st + p2cX(df$radius + df$offset, df$angle)
+			y.lab <- y.st + p2cY(df$radius + df$offset, df$angle)		
+			}
+
+		if (anCoord == "global") {					
+			x.end <- p2cX(df$radius, df$angle)
+			y.end <- p2cY(df$radius, df$angle)
+						
+			x.lab <- p2cX(df$radius + df$offset, df$angle)
+			y.lab <- p2cY(df$radius + df$offset, df$angle)		
+			}
 		
-		grid.segments(x0 = x.st, x1 = x.end, y0 = y.st, y1 = y.end,
-			default.units = "native", gp = anNode.gpar)
-		grid.text(ann$node.text, x.lab, y.lab, hjust = ann$hjust, vjust = ann$vjust,
-			default.units = "native", gp = anNode.gpar)
+		retval <- data.frame(x.st, y.st, x.end, y.end, x.lab, y.lab)
+		retval
 		}
 
 ###############
@@ -252,7 +307,8 @@ plotHive <- function(HPD, ch = 1, method = "abs",
 	# Add a legend arrow & any annotations
 		
 		if (!is.null(arrow)) addArrow(arrow, nx)
-		if (!is.null(anNodes)) annotateNodes(anNodes, nodes, nx)
+		if (!is.null(anNodes)) annotateNodes(anNodes, nodes, nx, anCoord)
+		if (!is.null(grInfo)) addGraphic(grInfo, nodes, nx, anCoord)
 
 		} # end of 2D
 	
@@ -543,7 +599,8 @@ plotHive <- function(HPD, ch = 1, method = "abs",
 	# Add a legend arrow & any annotations
 		
 		if (!is.null(arrow)) addArrow(arrow, nx)
-		if (!is.null(anNodes)) annotateNodes(anNodes, nodes, nx)
+		if (!is.null(anNodes)) annotateNodes(anNodes, nodes, nx, anCoord)
+		if (!is.null(grInfo)) addGraphic(grInfo, nodes, nx, anCoord)
 
 		} # end of 3D
 	
@@ -896,7 +953,8 @@ plotHive <- function(HPD, ch = 1, method = "abs",
 	# Add a legend arrow & any annotations
 		
 		if (!is.null(arrow)) addArrow(arrow, nx)
-		if (!is.null(anNodes)) annotateNodes(anNodes, nodes, nx)
+		if (!is.null(anNodes)) annotateNodes(anNodes, nodes, nx, anCoord)
+		if (!is.null(grInfo)) addGraphic(grInfo, nodes, nx, anCoord)
 
 		} # end of 4D
 	
@@ -1316,7 +1374,8 @@ plotHive <- function(HPD, ch = 1, method = "abs",
 	# Add a legend arrow & any annotations
 		
 		if (!is.null(arrow)) addArrow(arrow, nx)
-		if (!is.null(anNodes)) annotateNodes(anNodes, nodes, nx)
+		if (!is.null(anNodes)) annotateNodes(anNodes, nodes, nx, anCoord)
+		if (!is.null(grInfo)) addGraphic(grInfo, nodes, nx, anCoord)
 
 		} # end of 5D
 
@@ -1796,7 +1855,8 @@ plotHive <- function(HPD, ch = 1, method = "abs",
 	# Add a legend arrow & any annotations
 		
 		if (!is.null(arrow)) addArrow(arrow, nx)
-		if (!is.null(anNodes)) annotateNodes(anNodes, nodes, nx)
+		if (!is.null(anNodes)) annotateNodes(anNodes, nodes, nx, anCoord)
+		if (!is.null(grInfo)) addGraphic(grInfo, nodes, nx, anCoord)
 
 		} # end of 6D
 
